package cn.com.qmth.examcloud.core.oe.student.controller;

import cn.com.qmth.examcloud.api.commons.security.bean.User;
import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.oe.common.base.Constants;
import cn.com.qmth.examcloud.core.oe.common.entity.ExamRecordDataEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.FaceBiopsyEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.FaceBiopsyItemEntity;
import cn.com.qmth.examcloud.core.oe.common.entity.FaceBiopsyItemStepEntity;
import cn.com.qmth.examcloud.core.oe.common.enums.FaceBiopsyType;
import cn.com.qmth.examcloud.core.oe.common.repository.ExamRecordDataRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.FaceBiopsyItemRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.FaceBiopsyItemStepRepo;
import cn.com.qmth.examcloud.core.oe.common.repository.FaceBiopsyRepo;
import cn.com.qmth.examcloud.core.oe.student.bean.ExamSessionInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.FaceBiopsyInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.FaceBiopsyStepInfo;
import cn.com.qmth.examcloud.core.oe.student.bean.SaveFaceBiopsyResultResp;
import cn.com.qmth.examcloud.core.oe.student.bean.SaveFaceBiopsyResultReq;
import cn.com.qmth.examcloud.core.oe.student.service.ExamSessionInfoService;
import cn.com.qmth.examcloud.core.oe.student.service.FaceBiopsyService;
import cn.com.qmth.examcloud.web.helpers.GlobalHelper;
import cn.com.qmth.examcloud.web.helpers.SequenceLockHelper;
import cn.com.qmth.examcloud.web.support.ControllerSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * @Description 人脸活体检测接口
 * @Author lideyin
 * @Date 2019/10/14 10:59
 * @Version 1.0
 */
@Api(tags = "人脸活体检测")
@RestController
@RequestMapping("${app.api.oe.student}/faceBiopsy")
public class FaceBiopsyController extends ControllerSupport {
    @Autowired
    private ExamSessionInfoService examSessionInfoService;
    @Autowired
    private FaceBiopsyService faceBiopsyService;
    @Autowired
    private FaceBiopsyRepo faceBiopsyRepo;
    @Autowired
    private FaceBiopsyItemRepo faceBiopsyItemRepo;
    @Autowired
    private FaceBiopsyItemStepRepo faceBiopsyItemStepRepo;
    @Autowired
    private ExamRecordDataRepo examRecordDataRepo;

    @ApiOperation(value = "获取人脸活体检测基本信息")
    @GetMapping("/getFaceBiopsyInfo")
    public FaceBiopsyInfo getFaceBiopsyInfo(@RequestParam Long examRecordDataId) {
        User user = getAccessUser();
        Long studentId = user.getUserId();
        String sequenceLockKey = Constants.GET_FACE_BIOPSY_INFO_PREFIX + studentId;
        //系统在请求结束后会，自动释放锁，无需手动解锁
        SequenceLockHelper.getLock(sequenceLockKey);

        //判断考试记录id是否有效
        ExamRecordDataEntity examRecordData = GlobalHelper.getEntity(examRecordDataRepo, examRecordDataId,
                ExamRecordDataEntity.class);
        if (examRecordData == null) {
            throw new StatusException("200101", "无效的考试记录");
        }

        // 获取考试会话，判断考生是否已结束考试
        ExamSessionInfo examSessionInfo = examSessionInfoService.getExamSessionInfo(studentId);
        if (examSessionInfo == null) {
            throw new StatusException("200102", "考试会话已过期");
        }

        return faceBiopsyService.getFaceBiopsyInfo(user.getRootOrgId(), examRecordDataId, FaceBiopsyType.FACE_MOTION);
    }


    @ApiOperation(value = "保存活体检测结果")
    @PostMapping("/saveFaceBiopsyResult")
    public SaveFaceBiopsyResultResp saveFaceBiopsyResult(@RequestBody SaveFaceBiopsyResultReq req) {
        SaveFaceBiopsyResultResp result = new SaveFaceBiopsyResultResp();
        return result;
    }
}
